using Adaptation.Shared;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace Adaptation.FileHandlers.pcl;

#nullable enable

internal class Run
{

    public Run(Header header, ReadOnlyCollection<Wafer> wafers)
    {
        Header = header;
        Wafers = wafers;
    }

    public Header Header { get; }
    public ReadOnlyCollection<Wafer> Wafers { get; }

    private static void WriteJson(Logistics logistics, List<FileInfo> fileInfoCollection, Run result)
    {
        FileInfo fileInfo = new($"{logistics.ReportFullPath}.json");
        string json = JsonSerializer.Serialize(result, RunSourceGenerationContext.Default.Run);
        File.WriteAllText(fileInfo.FullName, json);
        File.SetLastWriteTime(fileInfo.FullName, logistics.DateTimeFromSequence);
        fileInfoCollection.Add(fileInfo);
    }

    private static ReadOnlyCollection<string> GetLines(JsonElement[]? jsonElements)
    {
        List<string> results = new();
        int columns = 0;
        StringBuilder stringBuilder = new();
        for (int i = 0; i < jsonElements?.Length;)
        {
            foreach (JsonProperty jsonProperty in jsonElements[0].EnumerateObject())
            {
                columns += 1;
                _ = stringBuilder.Append('"').Append(jsonProperty.Name).Append('"').Append('\t');
            }
            break;
        }
        if (jsonElements?.Length != 0)
            _ = stringBuilder.Remove(stringBuilder.Length - 1, 1);
        results.Add(stringBuilder.ToString());
        for (int i = 0; i < jsonElements?.Length; i++)
        {
            _ = stringBuilder.Clear();
            foreach (JsonProperty jsonProperty in jsonElements[i].EnumerateObject())
            {
                if (jsonProperty.Value.ValueKind == JsonValueKind.Object)
                    _ = stringBuilder.Append('\t');
                else if (jsonProperty.Value.ValueKind != JsonValueKind.String)
                    _ = stringBuilder.Append(jsonProperty.Value).Append('\t');
                else
                    _ = stringBuilder.Append('"').Append(jsonProperty.Value).Append('"').Append('\t');
            }
            _ = stringBuilder.Remove(stringBuilder.Length - 1, 1);
            results.Add(stringBuilder.ToString());
        }
        return results.AsReadOnly();
    }

    private static void WriteTabSeparatedValues(Logistics logistics, List<FileInfo> fileInfoCollection, Run run)
    {
        List<Row> results = new();
        Row row;
        FileInfo fileInfo = new($"{logistics.ReportFullPath}.tsv");
        for (int i = 0; i < run.Wafers.Count; i++)
        {
            row = new(run, i);
            results.Add(row);
        }
        string json = JsonSerializer.Serialize(results);
        JsonElement[]? jsonElements = JsonSerializer.Deserialize<JsonElement[]>(json);
        ReadOnlyCollection<string> lines = GetLines(jsonElements);
        File.WriteAllText(fileInfo.FullName, string.Join(Environment.NewLine, lines));
        File.SetLastWriteTime(fileInfo.FullName, logistics.DateTimeFromSequence);
        fileInfoCollection.Add(fileInfo);
    }

    private static ReadOnlyCollection<string> GetWaferIds(Header header)
    {
        List<string> results = new();
        foreach (WaferSummary waferSummary in header.WaferSummary)
            results.Add(waferSummary.Id);
        return results.AsReadOnly();
    }

    internal static Run? Get(Logistics logistics, List<FileInfo> fileInfoCollection, ReadOnlyDictionary<string, string> pages)
    {
        Run? result;
        Constant constant = new();
        string headerFileName = pages.ElementAt(pages.Count - 1).Key;
        Header? header = Header.Get(pages, constant, headerFileName);
        if (header is null)
            result = null;
        else
        {
            ReadOnlyCollection<string> waferIds = GetWaferIds(header);
            ReadOnlyDictionary<string, Wafer> keyValuePairs = Wafer.Get(pages, constant, headerFileName);
            ReadOnlyCollection<Wafer> wafers = Wafer.Get(waferIds, keyValuePairs);
            result = new(header, wafers);
            WriteJson(logistics, fileInfoCollection, result);
            WriteTabSeparatedValues(logistics, fileInfoCollection, result);
        }
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(Run))]
internal partial class RunSourceGenerationContext : JsonSerializerContext
{
}